/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.sla;

import org.ogf.graap.wsag.api.pattern.AdvanceReservationPattern;
import org.ogf.graap.wsag4j.types.scheduling.TimeConstraintType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;

//@formatter:off
/**
 * AdvanceReservationComputeJobTemplate
 * 
 * <pre>
 * {@code
 * <wsag:AgreementTemplateType wsag:TemplateId="1"
 *     xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *     <wsag:Name>ADVANCE-RESERVATION-COMPUTE-JOB</wsag:Name>
 *     <wsag:Context>
 *         <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *         <wsag:TemplateId>1</wsag:TemplateId>
 *         <wsag:TemplateName>ADVANCE-RESERVATION-COMPUTE-JOB</wsag:TemplateName>
 *     </wsag:Context>
 *     <wsag:Terms>
 *         <wsag:All>
 *             <wsag:ExactlyOne>
 *                 <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                     wsag:ServiceName="COMPUTE_SERVICE">
 *                     <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                         <jsdl:JobDescription>
 *                             <jsdl:Application>
 *                                 <jsdl:ApplicationName>Application_1</jsdl:ApplicationName>
 *                                 <jsdl:ApplicationVersion>1</jsdl:ApplicationVersion>
 *                                 <jsdl:Description>Description</jsdl:Description>
 *                             </jsdl:Application>
 *                         </jsdl:JobDescription>
 *                     </jsdl:JobDefinition>
 *                 </wsag:ServiceDescriptionTerm>
 *                 <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                     wsag:ServiceName="COMPUTE_SERVICE">
 *                     <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                         <jsdl:JobDescription>
 *                             <jsdl:Application>
 *                                 <jsdl:ApplicationName>Application_2</jsdl:ApplicationName>
 *                                 <jsdl:ApplicationVersion>2</jsdl:ApplicationVersion>
 *                                 <jsdl:Description>Description</jsdl:Description>
 *                             </jsdl:Application>
 *                         </jsdl:JobDescription>
 *                     </jsdl:JobDefinition>
 *                 </wsag:ServiceDescriptionTerm>
 *             </wsag:ExactlyOne>
 *             <wsag:ServiceDescriptionTerm wsag:Name="RESOURCE_SDT"
 *                 wsag:ServiceName="RESOURCE_PROVISIONING_SERVICE">
 *                 <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                     <jsdl:JobDescription>
 *                         <jsdl:Resources>
 *                             <jsdl:IndividualCPUTime>
 *                                 <jsdl:Exact>
 *                                     $RESOURCES.getIndividualCPUTime().getExactArray(0).
 *                                     getDoubleValue()</jsdl:Exact>
 *                             </jsdl:IndividualCPUTime>
 *                             <jsdl:IndividualCPUCount>
 *                                 <jsdl:Exact>
 *                                     $RESOURCES.getIndividualCPUCount().getExactArray(0).
 *                                     getDoubleValue()</jsdl:Exact>
 *                             </jsdl:IndividualCPUCount>
 *                             <jsdl:TotalResourceCount>
 *                                 <jsdl:Exact>
 *                                     $RESOURCES.getTotalResourceCount().getExactArray(0).
 *                                     getDoubleValue()</jsdl:Exact>
 *                             </jsdl:TotalResourceCount>
 *                         </jsdl:Resources>
 *                     </jsdl:JobDescription>
 *                 </jsdl:JobDefinition>
 *             </wsag:ServiceDescriptionTerm>
 *             <wsag:ServiceDescriptionTerm wsag:Name="TIME_CONSTRAINT_SDT"
 *                 wsag:ServiceName="RESERVATION_SERVICE">
 *                 <wsag4jt:TimeConstraint
 *                     xmlns:wsag4jt="http://schemas.wsag4j.org/2009/07/wsag4j-scheduling-extensions">
 *                     <wsag4jt:StartTime>2010-05-20T00:00:00</wsag4jt:StartTime>
 *                     <wsag4jt:Duration>1</wsag4jt:Duration>
 *                 </wsag4jt:TimeConstraint>
 *             </wsag:ServiceDescriptionTerm>
 *         </wsag:All>
 *     </wsag:Terms>
 *     <wsag:CreationConstraints> ... </wsag:CreationConstraints>
 * </wsag:AgreementTemplateType>
 * }
 * </pre> 
 * 
 * @author Oliver Waeldrich
 * 
 */

//@formatter:on
public class AdvanceReservationComputeJobTemplate extends AbstractComputeJobTemplate
{

    /**
     * Template name for which this pattern applies.
     */
    public static final String NAME = "ADVANCE-RESERVATION-COMPUTE-JOB";

    /**
     * Template id for which this pattern applies.
     */
    public static final String ID = "1";

    private final AdvanceReservationPattern arTemplate;

    /**
     * @param template
     *            the agreement template XML representation
     */
    public AdvanceReservationComputeJobTemplate( AgreementTemplateType template )
    {
        super( template );

        //
        // use the internal XML object for creating the pattern
        //
        template = getXMLObject();

        arTemplate = new AdvanceReservationPattern( template.getTerms().getAll() );
    }

    /**
     * @return the specified time constraints
     * 
     * @see AdvanceReservationPattern#getTimeConstraint()
     */
    public TimeConstraintType getTimeConstraint()
    {
        return arTemplate.getTimeConstraint();
    }

    /**
     * 
     * @return an offer based on this template
     * 
     * @throws Exception
     *             failed to create the offer object
     */
    public AdvanceReservationComputeJobOffer getOffer() throws Exception
    {
        return new AdvanceReservationComputeJobOffer( getXMLObject() );
    }
}
