/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.api.sla;

import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;

/**
 * ComputeJobTemplate
 * 
 * <pre>
 * {@code
 * <wsag:Template wsag:TemplateId="1"
 *  xmlns:wsag="http://schemas.ggf.org/graap/2007/03/ws-agreement">
 *  <wsag:Name>COMPUTE-JOB</wsag:Name>
 *  <wsag:Context>
 *      <wsag:ServiceProvider>AgreementResponder</wsag:ServiceProvider>
 *      <wsag:TemplateId>1</wsag:TemplateId>
 *      <wsag:TemplateName>COMPUTE-JOB</wsag:TemplateName>
 *  </wsag:Context>
 *  <wsag:Terms>
 *      <wsag:All>
 *          <wsag:ExactlyOne>
 *              <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                  wsag:ServiceName="COMPUTE_SERVICE">
 *                  <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                      <jsdl:JobDescription>
 *                          <jsdl:Application>
 *                              <jsdl:ApplicationName>Application_1</jsdl:ApplicationName>
 *                              <jsdl:ApplicationVersion>1</jsdl:ApplicationVersion>
 *                              <jsdl:Description>Description</jsdl:Description>
 *                          </jsdl:Application>
 *                      </jsdl:JobDescription>
 *                  </jsdl:JobDefinition>
 *              </wsag:ServiceDescriptionTerm>
 *              <wsag:ServiceDescriptionTerm wsag:Name="APPLICATION_SDT"
 *                  wsag:ServiceName="COMPUTE_SERVICE">
 *                  <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                      <jsdl:JobDescription>
 *                          <jsdl:Application>
 *                              <jsdl:ApplicationName>Application_2</jsdl:ApplicationName>
 *                              <jsdl:ApplicationVersion>2</jsdl:ApplicationVersion>
 *                              <jsdl:Description>Description</jsdl:Description>
 *                          </jsdl:Application>
 *                      </jsdl:JobDescription>
 *                  </jsdl:JobDefinition>
 *              </wsag:ServiceDescriptionTerm>
 *          </wsag:ExactlyOne>
 *          <wsag:ServiceDescriptionTerm wsag:Name="RESOURCE_SDT"
 *              wsag:ServiceName="RESOURCE_PROVISIONING_SERVICE">
 *              <jsdl:JobDefinition xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl">
 *                  <jsdl:JobDescription>
 *                      <jsdl:Resources>
 *                          <jsdl:IndividualCPUTime>
 *                              <jsdl:Exact>$RESOURCES.getIndividualCPUTime().
 *                              getExactArray(0).getDoubleValue()</jsdl:Exact>
 *                          </jsdl:IndividualCPUTime>
 *                          <jsdl:IndividualCPUCount>
 *                              <jsdl:Exact>$RESOURCES.getIndividualCPUCount().
 *                              getExactArray(0).getDoubleValue()</jsdl:Exact>
 *                          </jsdl:IndividualCPUCount>
 *                          <jsdl:TotalResourceCount>
 *                              <jsdl:Exact>$RESOURCES.getTotalResourceCount().
 *                              getExactArray(0).getDoubleValue()</jsdl:Exact>
 *                          </jsdl:TotalResourceCount>
 *                      </jsdl:Resources>
 *                  </jsdl:JobDescription>
 *              </jsdl:JobDefinition>
 *          </wsag:ServiceDescriptionTerm>
 *      </wsag:All>
 *  </wsag:Terms>
 *  <wsag:CreationConstraints> 
 *     ...
 *     </wsag:CreationConstraints>
 * </wsag:Template>
 * }
 * </pre>
 * 
 * @author Oliver Waeldrich
 * 
 */
public class ComputeJobTemplate extends AbstractComputeJobTemplate
{

    private static final String DEFAULT_TEMPLATE_NAME = "COMPUTE-JOB";

    private static final String DEFAULT_TEMPLATE_ID = "1";

    /**
     * @param template
     *            the agreement template XML representation
     */
    public ComputeJobTemplate( AgreementTemplateType template )
    {
        super( template );
    }

    /**
     * Creates an offer based on this template.
     * 
     * @return the new offer
     * @throws Exception
     *             failed to create the offer object
     */
    public ComputeJobOffer getOffer() throws Exception
    {
        return new ComputeJobOffer( getXMLObject() );
    }

    /**
     * @return the template name to which this pattern applies
     */
    public static String getDefaultTemplateName()
    {
        return DEFAULT_TEMPLATE_NAME;
    }

    /**
     * @return the template id to which this pattern applies
     */
    public static String getDefaultTemplateId()
    {
        return DEFAULT_TEMPLATE_ID;
    }

}
